# Generated by Django 4.2.13 on 2024-05-21 05:17

from django.db import migrations, models
import ellis_django_views.tests.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestImageModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', models.ImageField(upload_to=ellis_django_views.tests.models.image_directory)),
                ('datetime', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]
